// inputs.cc is part of the 
// CHIRON ChPT at two loops program collection
// Copyright (C) 2014 Johan Bijnens, v1.0
// CHIRON is licenced under the GNU GPL version 2 or later,
// see COPYING for details.
// Please respect the Guidelines, see GUIDELINES for details.


// implementation of input classes
// and setting them to particular values

#include <iostream>
#include <iomanip>
#include <string>
#include <cmath>
#include "inputs.h"

physmass::~physmass(void){}

physmass::physmass(const double mpiin, const double mkin,
		   const double metain,  const double fpiin,
		   const double muin){
  mpi = mpiin;
  mk = mkin;
  meta = metain;
  fpi = fpiin;
  mu = muin;
}

void physmass::setmpi(const double mpiin){
  mpi = mpiin;
}
void physmass::setmk(const double mkin){
  mk = mkin;
}
void physmass::setmeta(const double metain){
  meta = metain;
}
void physmass::setfpi(const double fpiin){
  fpi = fpiin;
}
void physmass::setmu(const double muin){
  mu = muin;
}

void physmass::out(double &mpiout, double &mkout, double &metaout,
		   double &fpiout, double &muout) const{
  mpiout = mpi;
  mkout = mk;
  metaout = meta;
  fpiout = fpi;
  muout = mu;
}

double physmass::getmpi(void ) const{
  return mpi;
}
double physmass::getmk(void) const{
  return mk;
}
double physmass::getmeta(void) const{
  return meta;
}
double physmass::getfpi(void) const{
  return fpi;
}
double physmass::getmu(void) const{
  return mu;
}

std::ostream & operator<<(std::ostream & os, const physmass & bb){
  os   << setprecision(15) << fixed
       << "# mpi :  "<< bb.mpi << "\n"
       << "# mk  :  "<<bb.mk <<"\n"
       << "# meta:  "<<bb.meta <<"\n"
       << "# fpi :  "<<bb.fpi <<"\n"
       << "# mu  :  "<<bb.mu <<"\n";
  os.unsetf(std::ios_base::floatfield);
  return os;
}

std::istream & operator>>(std::istream & is, physmass & mass){
  string temps;// used to sneak to the next line and check correct input
  // strings should be the same as the ones in mass output
  // up to the :
  string temps2;
  double mpi,mk,meta,fpi,mu;
  std::getline(is,temps2,':');// reads in characters up to ':'
  is >> mpi;
  std::getline(is,temps); // to remove end of line
  if (temps2 != "# mpi ") std::cout << "trouble reading in mass mpi "<< temps2<<'\n';
  std::getline(is,temps2,':');
  is >> mk;
  std::getline(is,temps);
  if (temps2 != "# mk  ") std::cout << "trouble reading in mass mk "<< temps2<<'\n';
  std::getline(is,temps2,':');
  is >> meta;
  std::getline(is,temps);
  if (temps2 != "# meta") std::cout <<"trouble reading in mass meta "<< temps2<<'\n';
  std::getline(is,temps2,':');
  is >> fpi;
  std::getline(is,temps);
  if (temps2 != "# fpi ") std::cout <<"trouble reading in mass fpi "<< temps2<<'\n';
  std::getline(is,temps2,':');
  is >> mu;
  std::getline(is,temps);
  if (temps2 != "# mu  ") std::cout << "trouble reading in mass mu "<< temps2<<'\n';
  mass = physmass(mpi,mk,meta,fpi,mu);
  return is;
}

bool operator==(const physmass mass1,const physmass mass2){
  const double massprecision = 1e-7;
  if ( (abs((mass1.mpi-mass2.mpi)/mass1.mpi) < massprecision) &&
       (abs((mass1.mk-mass2.mk)/mass1.mk) < massprecision) &&
       (abs((mass1.meta-mass2.meta)/mass1.meta) < massprecision) &&
       (abs((mass1.fpi-mass2.fpi)/mass1.fpi) < massprecision) &&
       (abs((mass1.mu-mass2.mu)/mass1.mu) < massprecision) ) return true;
  return false;
}

