// testinputs.cc is part of the CHIRON ChPT at two loops program collection
// Copyright (C) 2014 Johan Bijnens, v1.0
// CHIRON is licenced under the GNU GPL version 2 or later,
// see COPYING for details.
// Please respect the Guidelines, see GUIDELINES for details.

// test the masses etc in and output routines in inputs.cc

#include <iostream>
#include <iomanip>
#include <fstream>
#include <cmath>
using namespace std;

#include "inputs.h"

int main(void){
  // a number of different input values
  double  mpi = 0.15;
  double  mk = 0.5;
  double  meta = 0.55;
  double fpi = 0.95;
  double  mu = 0.9;

  // creating a set with standard input values and puttin it out;
  physmass mass1;
  cout << "First mass case\n" << mass1;
  // creating a set with the other inputs above and then changin one
  physmass mass2(mpi,mk,meta,fpi,mu);
  cout << "Second mass case\n" << mass2;
  physmass mass3 = mass2;
  mass3.setmk(0.4);
  cout << "Third mass case\n" << mass3;
  // writing it out to a file and then reading back in
  ofstream ouf("test.dat");
  ouf << mass2;
  ouf.close();
  physmass mass4;
  ifstream inf("test.dat");
  inf >> mass4;
  inf.close();
  cout << "written to and read from file\n" << mass4;
  // readin out the data
  mass4.out(mpi,mk,meta,fpi,mu);
  cout <<"mpi,mk,meta,fpi,mu\n"
       <<mpi<<' '<<mk<<' '<<meta<<' '<<fpi<<' '<<mu<<endl;
  // same but now read out with the separate fnuctions, resetting fpi
  mass4.setfpi(0.12);
  mpi = mass4.getmpi();
  mk  = mass4.getmk();
  meta= mass4.getmeta();
  fpi = mass4.getfpi();
  mu  = mass4.getmu();
  cout <<"mpi,mk,meta,fpi,mu\n"
       <<mpi<<' '<<mk<<' '<<meta<<' '<<fpi<<' '<<mu<<endl;
  // checking equality of two masses (note only to about 7 digits
  // to avoid calculated masses to become unequal
  if( mass1 == mass4){
    cout << "mass4 equal to mass1\n";}
  else{
    cout << "mass4 not equal to mass1\n";}
  physmass mass5;
  if( mass1 == mass5){
    cout << "mass5 equal to mass1\n";}
  else{
    cout << "mass5 not equal to mass1\n";}

  return 0;
}
